from typing import Optional, TypeVar, Union
import os


__version__ = "0.4.0"


T = TypeVar("T")


class _RaiseExceptionSentinel:
    """Sentinel indicating that an exception should be raised."""
    def __repr__(self):
        return "RAISE"

RAISE = _RaiseExceptionSentinel()


FALSY_STRINGS = {"0", "false", "f", "no", "n"}


class EnvironmentVariableNotSet(Exception):
    def __init__(self, varname: str):
        self.varname: str = varname

    def __str__(self):
        return f"{self.__class__.__name__}({self.varname!r})"


def get_int(varname: str, default: Union[T, _RaiseExceptionSentinel]=RAISE) -> Union[T, int]:
    if varname in os.environ:
        return int(os.environ[varname])
    if default is RAISE:
        raise EnvironmentVariableNotSet(varname)
    return default


def get_float(varname, default: Union[T, _RaiseExceptionSentinel]=RAISE) -> Union[T, float]:
    if varname in os.environ:
        return float(os.environ[varname])
    if default is RAISE:
        raise EnvironmentVariableNotSet(varname)
    return default


def get_bool(varname, default: Union[T, _RaiseExceptionSentinel]=RAISE, falsy_strings: Optional[set[str]]=None) -> Union[T, bool]:
    if falsy_strings is None:
        falsy_strings = FALSY_STRINGS
    if varname in os.environ:
        return os.environ[varname].lower() not in falsy_strings
    if default is RAISE:
        raise EnvironmentVariableNotSet(varname)
    return default


def get_str(varname, default: Union[T, _RaiseExceptionSentinel]=RAISE) -> Union[T, str]:
    if varname in os.environ:
        return os.environ[varname]
    if default is RAISE:
        raise EnvironmentVariableNotSet(varname)
    return default


__all__ = ["EnvironmentVariableNotSet", "get_int", "get_float", "get_bool", "get_str", "FALSY_STRINGS"]



from setuptools import setup

# https://setuptools.readthedocs.io/en/latest/
setup(
    name="envil",
    version="0.4.0",
    description="Get non-string values from environment variables.",
    long_description=open("README.md", "rt").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/luismsgomes/envil",
    author="Luís Gomes",
    author_email="luismsgomes@gmail.com",
    license="GPLv3",
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3.8",
    ],
    keywords="util",
    install_requires=[],
    package_dir={"": "src"},
    py_modules=["envil"],
)
